'''
Ported from original C at
http://playtechs.blogspot.com/2007/03/raytracing-on-grid.html
'''

BLOCKS = {(-1, 0): 'los-blocks-left',
          (1, 0): 'los-blocks-right',
          (0, 1): 'los-blocks-up',
          (0, -1): 'los-blocks-down'}
# assumes that the cell coordinates are integers
def line_of_sight(level, source, destination, ignore_creatures, range=None):

    if ignore_creatures:
       fetcher = level.ground.get_cell
    else:
       fetcher = level.get_cell

    if isinstance(source, tuple):
        x0, y0 = source[0], source[1]
    else:
        x0, y0 = source.x, source.y
    if isinstance(destination, tuple):
        x1, y1 = destination[0], destination[1]
    else:
        x1, y1 = destination.x, destination.y

    x, y = x0, y0
    dx, dy = abs(x1-x0), abs(y1-y0)

    if x1 > x0:
        x_inc =  1
    else:
        x_inc = -1

    if y1 > y0:
        y_inc =  1
    else:
        y_inc = -1

    n = 1 + dx + dy
    error = dx - dy

    dx *= 2; dy *= 2

    visited = []

    while n > 0:
        if visited:
            cur = fetcher(x, y)
            if cur:
                last = visited[-1]
                dir = (cur.i-last.i, cur.j-last.j)
                if last.get(BLOCKS[dir]):
                    visited = []
                    break
        visited.append(fetcher(x,y))

        if error > 0:
            x += x_inc
            error -= dy
        else:
            y += y_inc
            error += dx
        n -= 1

    return filter(None, visited)

