'''TEAM XKCD's AWESOME GAME.
'''

import os, shutil

import pyglet
import cocos
from cocos.director import director
from cocos.menu import *
from cocos.actions import *

from gamelib import party, world, cutscenes, DummyPlayer

def main():
    director.init(width=1024, height=768)
    director.show_FPS = True
#    director.window.set_exclusive_mouse(True)
    pyglet.gl.glClearColor(1, 1, 1, 1)
    pyglet.gl.glDisable(pyglet.gl.GL_DEPTH_TEST)

    director.run(cocos.scene.Scene(MainMenu("Threads of Fate")))


class MainMenu(Menu):
    def on_enter(self):
        super(MainMenu, self).on_enter()

        try:
            self.music_player = pyglet.media.Player()
            self.menu_music = pyglet.resource.media('music_menu_t.ogg')
        except pyglet.media.riff.WAVEFormatException:
            self.music_player = DummyPlayer()
            self.menu_music = None

        self.music_player.queue(self.menu_music)
        self.music_player.eos_action = self.music_player.EOS_LOOP
        self.music_player.play()

        self.menu_valign = CENTER
        self.menu_halign = CENTER

        self.font_item['color'] = (0, 0, 0, 255)
        self.font_item_selected['color'] = (150, 150, 0, 255)
        self.font_title['color'] = (0, 0, 0, 255)

        # ugh, menu should DO this
        for child in self.get_children():
            self.remove(child)

        options = [
            MenuItem('New Game', self.new_game),
            MenuItem('Story', self.story),
            MenuItem('Quit', self.on_quit),
        ]
        if os.path.exists('save_game'):
            options.insert(0, MenuItem('Load Game', self.load_game))

        self.create_menu(options)

    def on_exit(self):
        super(MainMenu, self).on_exit()
        # XXX fade volume down and fade menu out
        self.music_player.pause()
        self.music_player = None
        self.menu_music = None

    def load_game(self):
        savedir = 'save_game'
        p = party.Party.load(savedir)
        director.push(world.MapScene(savedir, p))
        # XXX director.push(cocos.scene.Scene(LoadMenu('Load Game')))

    def new_game(self):
        savedir = 'save_game'

        if os.path.exists(savedir):
            director.push(cocos.scene.Scene(ConfirmNuke('Destroy existing save game?')))
            return

        os.mkdir(savedir)
        p = party.Party.create(savedir)
        director.push(world.MapScene(savedir, p))

    def story(self):
        intro = cocos.scene.Scene(cutscenes.Intro())
        director.push(intro)

    def on_quit(self):
        pyglet.app.exit()


class ConfirmNuke(Menu):
    def on_enter(self):
        super(ConfirmNuke, self).on_enter()
        self.menu_valign = CENTER
        self.menu_halign = CENTER

        self.font_item['color'] = (0, 0, 0, 255)
        self.font_item_selected['color'] = (150, 150, 0, 255)
        self.font_title['color'] = (0, 0, 0, 255)

        # ugh, menu should DO this
        for child in self.get_children():
            self.remove(child)

        options = [
            MenuItem("Don't do it", self.on_no),
            MenuItem('Destroy old save game', self.on_yes),
        ]

        self.create_menu(options)

    def on_no(self):
        self.parent.end()

    def on_yes(self):
        savedir = 'save_game'
        shutil.rmtree(savedir)
        os.mkdir(savedir)

        p = party.Party.create(savedir)
        director.replace(world.MapScene(savedir, p))

