import random
from gamelib import character, namegen, pathfinding, actions, items, actions,\
        los

class NPC(character.Character):
    def __init__(self, *a, **kw):

        # grab the weights before invoking the base class
        if 'weights' in kw:
            self.weights = kw.pop('weights')
        else:
            self.weights = []

        super(NPC, self).__init__(*a, **kw)

        self.next_waypoint = self.position

    def figure_what_to_do(self, level):
        pass


class LameNPC(NPC):
    def figure_what_to_do(self, level):
        while self.actions_remaining:
            paths = filter(None, [pathfinding.find_path(level, self.position, c.position, ignore=(self, c))
                     for c in level.party.characters if (los.line_of_sight(level,
                                                            self.position,
                                                            c.position,
                                                            False) and
                                                         c.is_active())])
            paths.sort(lambda x, y: cmp(len(x.nodes), len(y.nodes)))
            if not paths:
                if self.position == self.next_waypoint:
                    nw = level.ground.get_cell(*self.position).get('next-waypoint')
                    if nw:
                        self.next_waypoint = tuple(map(int, nw.split(',')))
                path = pathfinding.find_path(level, self.position,
                                                   self.next_waypoint)
                if not path:
                    break
            else:
                path = paths[0]
            dest_node = path.nodes[0]
            cell = level.get_cell(dest_node.location[0], dest_node.location[1])
            thing = level.get_at_cell(dest_node.location[0],
                                      dest_node.location[1])
            if cell:
                acts = [a for a in self.get_action_options() if (a.type, a.name) in
                        self.weights]
                for act in acts:
                    if act.check(level, self, cell, thing) is True:
                        act.do()
                        break
                else:
                    break

implementations = dict(
    lame = LameNPC,
)

# per-NPC-class stats
class_defaults = dict(
    # the weakest of the lot
    kobold = dict(
        klass = 'lame',       # the class above to instantiate for this NPC
        strength = 5,
        dexterity = 5,
        constitution = 3,
        intelligence = 2,
        wisdom = 2,
        charisma = 2,
        max_health = 20,
        max_mana = 0,
        max_actions = 5,
        view_distance_dark = 30,    # haz low-light vision
        name = None,
        equipment = 'crap',
        carried = None,
        skills = dict(weapon_sword=5),
        weights = (
            ('attack', 'sword:sword'),
            ('attack', 'bash'),
            ('move', 'walk'),
        ),
        sprite_name = 'npc-red.png',
    ),

    # a little tougher
    goblin = dict(
        klass = 'lame',       # the class above to instantiate for this NPC
        strength = 6,
        dexterity = 5,
        constitution = 3,
        intelligence = 3,
        wisdom = 2,
        charisma = 2,
        max_health = 25,
        max_mana = 0,
        max_actions = 5,
        view_distance_dark = 30,    # haz low-light vision
        name = None,
        equipment = 'crap',
        carried = None,
        skills = dict(weapon_sword=5),
        weights = (
            ('attack', 'sword:sword'),
            ('attack', 'bash'),
            ('move', 'walk'),
        ),
        sprite_name = 'npc-green.png',
    ),
)

loadouts = dict(
    crap = dict(
        right_hand = items.Sword(quality=.5),
        chest = items.HideVest(quality=.75),
    ),
)

def create(cell):
    '''Create a new NPC using the meta-data from the cell.

    The cell will define an npc-class which gives us the base for the NPC. The
    cell may then also supply additional NPC properties to override the
    defaults.
    '''
    gender = random.choice(['male','female'])
    cclass = cell['npc-class']

    # figure the stats looking in the cell first and then the defaults
    info = {}
    defaults = class_defaults[cclass]
    for k in defaults:
        if k in cell:
            info[k] = cell[k]
        else:
            info[k] = defaults[k]

    # figure the NPC Python class to use
    klass = implementations[info.pop('klass')]

    # if a name wasn't supplied by the XML generate one
    name = info.pop('name')
    if name is None:
        name = namegen.kuruk()

    # starting equipment
    # XXX allow customisation of equipment lists in XML
    info['equipment'] = loadouts[info.pop('equipment')]

    # create the NPC
    return klass(gender, cclass, name, (cell.i, cell.j), **info)

