from gamelib import astar

def find_path(map, start, end, ignore=None):
    '''Find a good path from start to end on the map.

    map is a cocos.tiles.RectMap
    start and end are (i, j) cell location tuples
    '''
    # the astar code uses tuples for identifying nodes, so make sure the locations
    # are tuples
    assert isinstance(start, tuple)
    assert isinstance(end, tuple)
    return astar.AStar(LevelHandler(map, ignore)).findPath(start, end)

class LevelHandler:
    '''Solve paths in our game's Level.

    Sometimes we need to ignore the contents of a cell so we can make a
    path to it.
    '''

    def __init__(self, tilemap, ignore=None):
        self.m = tilemap
        self.ignore = ignore
        #print 'ignoring', self.ignore

    def getNode(self, location):
        ' location is just a tuple '
        return astar.Node(location, 1, location)

    def _handleNode(self, fromnode, cell, dest):
        x, y = cell.i, cell.j
        n = astar.Node((x, y), 1, (x, y))
        destx, desty = dest

        dx = max(x, destx) - min(x, destx)
        dy = max(y, desty) - min(y, desty)

        emCost = dx+dy
        n.mCost += fromnode.mCost
        n.score = n.mCost + emCost
        n.parent = fromnode
        return n

    def getAdjacentNodes(self, curnode, dest):
        map = self.m

        cell = map.get_cell(curnode.location[0], curnode.location[1],
            ignore=self.ignore)
        dl = map.get_cell(dest[0], dest[1], ignore=self.ignore)

        result = []
        #print 'getAdjacentNodes', cell

        n = map.get_neighbor(cell, map.LEFT, ignore=self.ignore)
        if n and not cell.get('blocks-left') and not n.get('impassable'):
            result.append(self._handleNode(curnode, n, dest))

        n = map.get_neighbor(cell, map.RIGHT, ignore=self.ignore)
        if n and not cell.get('blocks-right') and not n.get('impassable'):
            result.append(self._handleNode(curnode, n, dest))

        n = map.get_neighbor(cell, map.DOWN, ignore=self.ignore)
        if n and not cell.get('blocks-down') and not n.get('impassable'):
            result.append(self._handleNode(curnode, n, dest))

        n = map.get_neighbor(cell, map.UP, ignore=self.ignore)
        if n and not cell.get('blocks-up') and not n.get('impassable'):
            result.append(self._handleNode(curnode, n, dest))

        return result

